*****************************************************************************
*** FUNCTION phi calculates the flux above threshold (flxk=1,2 or 3) 
*** or the differential flux (flxk=101, 102 or 103) from a given
*** annihilation channel. Channels CH=1-12 are supported.
*** If flxk = 1 or 101 - neutrino fluxes are given
***           2 or 102 - muon distributions at creation are given
***           3 or 103 - muon fluxes at the detector are given
*** The units are 1e-30 m**-2 (annihilation)**-1 for 1 and 3, and
*** 1e-30 m**-3 (annihilation)**-1 for 2.
*** For the differential fluxes, the units
*** are the same plus GeV**-1 degree**-1.
***
*** Author: Joakim Edsjo (edsjo@physics.berkeley.edu)
*** Date: 1995
*** Modified: Dec 03, 1997
*****************************************************************************

      real*8 function MFphi(mneu,emuthr,thmax,ch,wh,flxk,istat)
      implicit none
      include 'MFCom.h'

C------------------------ Variables ------------------------------------

      real*8 mneu,emuthr,thmax
      real*8 mp1,mp2,e1,e2,flx
      integer ch,istat,flxk,fltyp
      character*2 wh

      logical chok

      logical first
      data first /.true./
      save first

C------------------------ Functions ------------------------------------

      real*8 MFphii,MFphih

C-----------------------------------------------------------------------

      istat=0
      MFerr=0
      if (flxk.lt.100) then
        fltyp=1      ! integrated fluxes
      else
        fltyp=2      ! differential fluxes
      endif

C--------------------------------------- If first call, load muon tables

      if (first) then
        flxtype(1)=0
        flxtype(2)=0
        first=.false.
        call MFinit(flxk)
      endif

      if ((flxk.ne.flxtype(1)).and.(flxk.ne.flxtype(2))) 
     &  then
        if ((flxk.lt.100.and.flxtype(1).ne.0).or.
     &    (flxk.gt.100.and.flxtype(2).ne.0)) then 
          write(*,*) 'WARNING in MFphi: You have requested a different',
     &     ' kind of flux than loaded. Reloading necessary.'
        endif
        call MFinit(flxk)
      endif

C-----------------------------------------------------------------------
      if (ch.le.6) then              ! "Fundamental" channel
        chok = .false.
        if (mneu.ge.map(ch)) chok = .true.
        if ((ch.eq.3.or.ch.eq.5.or.ch.eq.6).and.
     &     mneu.gt.(0.99*map(ch))) chok = .true.
        if (chok) then
          MFphi=MFphii(mneu,emuthr,thmax,ch,wh,fltyp,istat)
        else
          MFphi=0.0
          istat=mod(istat,2)+2
            write(6,5000) 'ERROR in MFphi: channel ',ch,' is not',
     &        ' energetically allowed.'
            write(6,*) 'Model: ',MFtag
        endif

      else                           ! "Complex" channel
        
C...Determine masses of the annihilation particles
        if (ch.eq.7) then
          mp1=mh(1)         ! H10 mass
          mp2=mh(3)         ! H30 mass
        elseif (ch.eq.8) then
          mp1=map(6)        ! Z0 mass
          mp2=mh(1)         ! H10 mass
        elseif (ch.eq.9) then
          mp1=map(6)        ! Z0 mass
          mp2=mh(2)         ! H20 mass
        elseif (ch.eq.10) then
          mp1=map(5)        ! W+- mass
          mp2=mh(4)         ! H+- mass
        elseif (ch.eq.11) then
          mp1=mh(2)         ! H20 mass
          mp2=mh(3)         ! H30 mass
        elseif (ch.eq.12) then
          mp1=map(6)        ! Z0 mass
          mp2=0.0d0         ! gamma mass
        endif

C...If energetically allowed channel, go on...
        if (mneu.ge.0.995d0*((mp1+mp2)/2.0d0)) then

C...Calculate the energy of the annihilation particles
          e1=((2.0d0*mneu)**2-mp2**2+mp1**2)/(4.0d0*mneu)
          e2=2.0d0*mneu-e1
          e1=max(e1,mp1+0.001d0)
          e2=max(e2,mp2+0.001d0)

C...Check different annihilation channels
          flx=0.0d0

C---------- H10 H30 channel ----------
          if (ch.eq.7) then
            flx=flx+MFphih(e1,emuthr,thmax,1,wh,fltyp,istat)
            flx=flx+MFphih(e2,emuthr,thmax,3,wh,fltyp,istat)
          endif
         
C---------- Z0 H10 channel ----------
          if (ch.eq.8) then
            flx=flx+0.5d0*MFphii(e1,emuthr,thmax,6,wh,fltyp,istat)
            flx=flx+MFphih(e2,emuthr,thmax,1,wh,fltyp,istat)
          endif

C---------- Z0 H20 channel ----------
          if (ch.eq.9) then
            flx=flx+0.5d0*MFphii(e1,emuthr,thmax,6,wh,fltyp,istat)
            flx=flx+MFphih(e2,emuthr,thmax,2,wh,fltyp,istat)
          endif

C---------- W+H- W-H+ channel ----------
C...This calculation gives a mean of the two channels W+H- & W-H+
          if (ch.eq.10) then
            flx=flx+0.5d0*MFphii(e1,emuthr,thmax,5,wh,fltyp,istat)
            flx=flx+MFphih(e2,emuthr,thmax,4,wh,fltyp,istat)
          endif

C---------- H20 H30 channel ----------
          if (ch.eq.11) then
            flx=flx+MFphih(e1,emuthr,thmax,2,wh,fltyp,istat)
            flx=flx+MFphih(e2,emuthr,thmax,3,wh,fltyp,istat)
          endif
         
C---------- Z0 gamma channel ----------
          if (ch.eq.12) then
            flx=flx+0.5d0*MFphii(e1,emuthr,thmax,6,wh,fltyp,istat)
          endif

          MFphi=flx

        else   ! Not energetically allowed channel
          MFphi=0.0d0
          istat=mod(istat,2)+2
c          if (MFprt.gt.0) then
            write(6,5000) 'ERROR in MFphi: channel ',ch,' is not',
     +        ' energetically allowed.'
            write(6,*) 'Model: ',MFtag
c          endif
        endif

      endif

      if (MFerr.gt.0) then
        write(*,*) 'WARNING in MFphi for model ',MFtag,
     &    ', flux ',flxk,' and channel ',ch
        write(*,*) '  The integration over Higgs decay angles ran',
     &    'into numerical problems.'
        write(*,*) '  The results can only be trusted as a lower',
     &    'bound.'
        MFistat=ibset(MFistat,2)
        istat=ibset(istat,2)
      endif

 5000 format(' ',a,i2,a,a)

      end





